unit Character;

interface

uses
  World;

const
  VelIncrement: Extended = 0.25;
//  JumpIncrement: Extended = 0.5;  ...obsolete...
  JumpImpulse: Extended = 6; //Realnija fizika, umesto JumpIncrement mehanike
  Friction: Extended = 0.25;
  Gravity: Extended = 0.25;
  MaxVel: Extended = 4;
  MaxJump: Extended = 60;

type
  TCharacter = class
  private
    //Indikatori su sada set...
    //Privatna polja premestena u public zbog debug-a u Form1
    //a i mrzelo me da pisem access methods za svako polje...
    //ionako cu ih vratiti u private kad sve bude kako treba
  public
    World: TWorld;
    X: Extended; //X Koordinata
    PriorX: Extended; //Prethodni (pouzdano "dobri" X)
    Y: Extended; //Y Koordinata
    PriorY: Extended;  //Prethodni (pouzdano "dobri" Y)
    H: Extended; //Trenutna Visina skoka u odnosu na StartY
                 //Verovatno ce i on biti obsolete...
                 //Jedini nacin dobijanja MaxH je empirijski za sada (75)...
    Vx: Extended;//X komponenta brzine
    Vy: Extended;//Y komponenta brzine
    StartY: Extended;  //Pocetni Y koji se pamti u trenutku skoka, bice obsolete...
    //Indikatori stanja
    Flags: set of (flgJumping, flgInAir, flgGoingLeft, flgGoingRight);
    constructor Create(AWorld: TWorld);
    procedure Tick; //Otkucaj, ciklus...
    procedure GoLeft; //Idi levo (pritisnut taster)
    procedure GoRight; //Idi desno (pritisnut taster)
    procedure GoJump; //Skoci (pritisnut taster)
    procedure StopLeft; //Stani levo (pusten taster)
    procedure StopRight; //Stani desno (pusten taster)
    procedure StopJump; //Stani sa skokom (pusten taster)

    function GetX: Integer; //Vraca trenutni X
    function GetY: Integer; //Vraca trenutni Y
    //Mozda ce ova 2 metoda biti 1 (TPoint ili sl.)
    //Btw, zna li neko kako se pravi union u Delphiu?
    //zbog mogucnosti pristupa (X i Y posebno) i (TPointu)
  end;

implementation

{ TCharacter }

constructor TCharacter.Create(AWorld: TWorld);
begin
  World := AWorld;
  X := 1;
  Y := 1;
  Vx := 0;
  Vy := 0;
  Flags := [];
end;

function TCharacter.GetX: Integer;
begin
  Result := Trunc(Int(X));
end;

function TCharacter.GetY: Integer;
begin
  Result := Trunc(Int(Y));
end;

procedure TCharacter.GoJump;
begin
  if not (flgInAir in Flags) then //Komentuj *ovaj* red za
  begin                           //efekat "letenja" tj.
    Vy := JumpImpulse;            //mahanja "krilima"
    Flags := Flags + [flgJumping];//interesantno uz neku
    Flags := Flags + [flgInAir];  //buducu HasWings proveru
  end;
end;

procedure TCharacter.GoLeft;
begin
  Flags := Flags + [flgGoingLeft];
  Flags := Flags - [flgGoingRight];
end;

procedure TCharacter.GoRight;
begin
  Flags := Flags - [flgGoingLeft];
  Flags := Flags + [flgGoingRight];
end;

procedure TCharacter.StopJump;
begin
  Flags := Flags - [flgJumping];
end;

procedure TCharacter.StopLeft;
begin
  Flags := Flags - [flgGoingLeft];
end;

procedure TCharacter.StopRight;
begin
  Flags := Flags - [flgGoingRight];
end;

procedure TCharacter.Tick;
begin
  if flgGoingRight in Flags then
  begin
    if Vx < MaxVel then
      Vx := Vx + VelIncrement;
    if Vx > MaxVel then
      Vx := MaxVel;
  end;

  if flgGoingLeft in Flags then
  begin
    if Vx > -MaxVel then
      Vx := Vx - VelIncrement;
    if Vx < -MaxVel then
      Vx := -MaxVel;
  end;

  if not ((flgGoingLeft in Flags) or (flgGoingRight in Flags)) then
    if Vx <> 0 then
      if Vx > 0 then
        Vx := Vx - Friction
      else
        Vx := Vx + Friction;

  if flgInAir in Flags then
    if H > 0  then
      Vy := Vy - Gravity;

  H := H + Vy;

  if H >= MaxJump then
  begin
    Flags := Flags - [flgJumping];
   // H := MaxJump; Odkomentuj da character udara glavom o MaxJump
  end;

  if H < 0 then
  begin
    Flags := Flags - [flgInAir];
    H := 0;
    Vy := 0;
  end;

  PriorX := X;
  PriorY := Y;
  X := X + Vx;
  Y := StartY + H;
  if Y < 1 then Y := 1; //ne znam za sada zasto ovo mora, ali mora...

  //Sledi pixel-precise provera kolizije sa nivoom
  //Bice obsolete zbog Srkijeve ideje sa tile-ovima

  //Vidi da li je nova koordinata slobodna
  if not World.IsAvailablePos(Trunc(Int(X)), Trunc(Int(Y))) then
    //Utvrditi koja koordinada ne valja: X, Y ili obe
    if World.IsAvailablePos(Trunc(Int(PriorX)), Trunc(Int(Y))) then
    begin
    //znaci ne valja X
      repeat
        if X < PriorX then
          X := X + VelIncrement
        else
          X := X - VelIncrement;
      until World.IsAvailablePos(Trunc(Int(X)), Trunc(Int(Y)));
      Vx := 0; //Drkni Vx na 0 da se ne bi igrac privremeno "zalepio" za objekat ako se zakuco maksimalnom brzinom i treba vremena da deluje frikcija
    end
    else if World.IsAvailablePos(Trunc(Int(X)), Trunc(Int(PriorY))) then
    //znaci ne valja Y
    begin
      repeat
        if Y < PriorY then
          Y := Y + VelIncrement
        else
          Y := Y - VelIncrement;
      until World.IsAvailablePos(Trunc(Int(X)), Trunc(Int(Y)));
      Vy := 0; //Isto ko Vx = 0;
    end
    else
    begin
    //znaci ne valjaju obe - jako retko
      repeat
        if X < PriorX then
          X := X + VelIncrement
        else
          X := X - VelIncrement;
        if Y < PriorY then
          Y := Y + VelIncrement
        else
          Y := Y - VelIncrement;
      until World.IsAvailablePos(Trunc(Int(X)), Trunc(Int(Y)));
      Vx := 0;
      Vy := 0;
    end;
end;


end.
